use crate::codec::ron::RonCodec;
use crate::models;
use crate::models::HoldPosition;
use crate::models::HoldRole;
use leptos::prelude::*;
use serde::Deserialize;
use serde::Serialize;

#[component]
pub fn Wall() -> impl leptos::IntoView {
    let load = async move {
        // TODO: What to do about this unwrap?
        load_initial_data().await.unwrap()
    };

    leptos::view! {
        <Await future=load let:data>
            <WallReady data=data.to_owned() />
        </Await>
    }
}

#[component]
fn WallReady(data: InitialData) -> impl leptos::IntoView {
    let mut hold_positions = vec![];
    for row in 0..(data.wall.rows) {
        for col in 0..(data.wall.cols) {
            hold_positions.push(HoldPosition { row, col });
        }
    }

    let (current_problem, current_problem_writer) = signal(None::<models::Problem>);
    LocalResource::new(move || async move {
        leptos::logging::log!("Loading random problem");
        let problem = get_random_problem().await.expect("cannot get random problem");
        current_problem_writer.set(Some(problem.into_inner()));
    });

    let mut cells = vec![];
    for &hold_position in &hold_positions {
        let role = move || {
            let x = current_problem
                .get()
                .map(|problem| {
                    let role = problem.holds.get(&hold_position)?;
                    // leptos::logging::log!("{hold_position:?}: {role:?}");
                    Some(*role)
                })
                .flatten();
            x
        };
        let role = Signal::derive(role);

        let cell = view! { <Cell role/> };
        cells.push(cell);
    }

    view! {
        <div class="container mx-auto border">
            <div class="grid grid-rows-4 grid-cols-12 gap-4">{cells}</div>
        </div>
    }
}

#[component]
fn Cell(#[prop(into)] role: Signal<Option<HoldRole>>) -> impl leptos::IntoView {
    let classes = move || {
        let role_classes = role.get().map(|role| match role {
            HoldRole::Start => "outline outline-offset-2 outline-green-500",
            HoldRole::Normal => "outline outline-offset-2 outline-blue-500",
            HoldRole::Zone => "outline outline-offset-2 outline-amber-500",
            HoldRole::End => "outline outline-offset-2 outline-red-500",
        });
        let mut s = "aspect-square rounded border-2 border-dashed border-sky-500 bg-indigo-100".to_string();
        if let Some(c) = role_classes {
            s.push_str(" ");
            s.push_str(c);
        }
        s
    };

    // let is_start = move || matches!(role.get(), Some(HoldRole::Start));
    // let is_normal = move || matches!(role.get(), Some(HoldRole::Normal));
    // let is_end = move || matches!(role.get(), Some(HoldRole::End));

    // let classes_start = "outline outline-offset-2 outline-green-500";

    view! {
        <div
           class=move || classes()

           // class="aspect-square rounded border-2 border-dashed border-sky-500 bg-indigo-100"
           // class=move || (is_start().then_some("outline outline-offset-2 outline-green-500"))
           // class=(["outline", "outline-offset-2", "outline-green-500"], is_start)
           // class=(["outline", "outline-offset-2", "outline-blue-500"], is_normal)
           // class=(["outline", "outline-offset-2", "outline-red-500"], is_end)
       >
            "o"
        </div>
    }

    // view! {
    //     {format!("{:?}", role)}
    // }
}

#[derive(Serialize, Deserialize, Clone)]
pub struct InitialData {
    wall: models::Wall,
}

#[server]
async fn load_initial_data() -> Result<InitialData, ServerFnError> {
    use crate::server::state::State;

    let state = expect_context::<State>();

    let wall = state.persistent.with(|s| s.wall.clone()).await;
    Ok(InitialData { wall })
}

#[server]
async fn get_random_problem() -> Result<RonCodec<models::Problem>, ServerFnError> {
    use crate::server::state::State;

    let state = expect_context::<State>();

    // TODO: Actually randomize
    let problem = state.persistent.with(|s| s.problems.problems.iter().next().unwrap().clone()).await;

    tracing::debug!("Returning randomized problem: {problem:?}");

    Ok(RonCodec::new(problem))
}
